Package: RLRsim
Type: Package
Title: Exact (Restricted) Likelihood Ratio Tests for Mixed and Additive
        Models
Version: 3.1-9
Authors@R: c(person("Fabian", "Scheipl", role = c("aut", "cre"),
	   email = "fabian.scheipl@stat.uni-muenchen.de",
	   comment=c(ORCID="0000-0001-8172-3603")),
	   person("Ben", "Bolker", role = "ctb",
	   comment=c(ORCID="0000-0002-2127-0443")))
Maintainer: Fabian Scheipl <fabian.scheipl@stat.uni-muenchen.de>
Description: Rapid, simulation-based exact (restricted) likelihood ratio tests
    for testing the presence of variance components/nonparametric terms for
    models fit with nlme::lme(),lme4::lmer(), lmerTest::lmer(), gamm4::gamm4() and
    mgcv::gamm().
License: GPL
URL: https://github.com/fabian-s/RLRsim
BugReports: https://github.com/fabian-s/RLRsim/issues
Depends: R (>= 2.14.0)
Imports: Rcpp (>= 0.11.1), lme4 (>= 1.1), mgcv, nlme
LinkingTo: Rcpp
Enhances: lmerTest
RoxygenNote: 7.1.2
NeedsCompilation: yes
Packaged: 2025-11-28 12:25:07 UTC; fabians
Author: Fabian Scheipl [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8172-3603>),
  Ben Bolker [ctb] (ORCID: <https://orcid.org/0000-0002-2127-0443>)
Repository: CRAN
Date/Publication: 2025-11-28 13:00:48 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 06:16:19 UTC; unix
Archs: RLRsim.so.dSYM
