# Generated by roxygen2: do not edit by hand


if(getRversion() >= "3.3.0") {
  importFrom("stats", sigma)
} else {
  importFrom("lme4", sigma)
}
export(LRTSim)
export(RLRTSim)
export(exactLRT)
export(exactRLRT)
export(extract.lmeDesign)
import(Rcpp)
importFrom(lme4,VarCorr)
importFrom(lme4,getME)
importFrom(mgcv,tensor.prod.model.matrix)
importFrom(nlme,getGroups)
importFrom(stats,anova)
importFrom(stats,coefficients)
importFrom(stats,complete.cases)
importFrom(stats,cov2cor)
importFrom(stats,formula)
importFrom(stats,logLik)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,quantile)
importFrom(stats,rchisq)
importFrom(utils,packageVersion)
useDynLib(RLRsim, .registration = TRUE)
