# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

RLRsimCpp <- function(p, k, n, nsim, g, q, mu, lambda, lambda0, xi, REML) {
    .Call(`_RLRsim_RLRsimCpp`, p, k, n, nsim, g, q, mu, lambda, lambda0, xi, REML)
}

